#include <QApplication>
#include <QDebug>
#include <QAction>
#include <QToolBar>
#include <QMenuBar>
#include <QMainWindow>
#include <QPlainTextEdit>
#include <QDesktopWidget>
#include <QMoveEvent>
#if QT_VERSION >= 0x050000
#  include <QWindow>
#  include <QScreen>
#endif
#ifdef QT_DIAG_LIB
#  include <eventfilter.h>
#endif

class MainWindow : public QMainWindow
{
    Q_OBJECT
public:
    MainWindow(QWidget *parent = 0);

public slots:
    void dumpClicked();
    void screensAdded() { qDebug() << __FUNCTION__ ; }
    void screensRemoved() { qDebug() << __FUNCTION__ ; }
    void workAreaResized(int i) { qDebug() << __FUNCTION__ << i; }
    void screenCountChanged(int i) { qDebug() << __FUNCTION__ << i; }
};

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
{
    setObjectName("Main");
    QMenu *menuApplication = menuBar()->addMenu(tr("&Application"));
    menuApplication->setObjectName("Menu");

    QAction *actDmp = menuApplication->addAction(tr("&Dump"));
    connect(actDmp, SIGNAL(triggered()), this, SLOT(dumpClicked()));
    actDmp->setShortcut(Qt::CTRL + Qt::Key_D);

    QAction *actApplicationExit = menuApplication->addAction(tr("&Exit"));
    connect(actApplicationExit, SIGNAL(triggered()), this, SLOT(close()));
    actApplicationExit->setShortcut(Qt::CTRL + Qt::Key_Q);

    QToolBar *t = addToolBar("F");

    t->setObjectName("ToolBar");
    t->addAction(actDmp);
    t->addAction(actApplicationExit);

    setWindowTitle(QT_VERSION_STR);
}


void MainWindow::dumpClicked()
{
    static int n= 0;

    QDebug d = qDebug();
    d << '#' << n++ << __FUNCTION__ << '\n';

    const int sc = QApplication::desktop()->screenCount();
    const int prim = QApplication::desktop()->primaryScreen();

    for (int i = 0; i < sc ; ++i ) {
        d << " - " << (i == prim ? '*' : ' ');
#if QT_VERSION >= 0x050000
        const QScreen *scr = QGuiApplication::screens().at(i);
        d << "QScreen" << scr->name() << scr->availableGeometry();
#endif
        d << "DesktopW" << QApplication::desktop()->availableGeometry(i) << '\n';
    }

    d << "DesktopW avg(-1)" << QApplication::desktop()->availableGeometry(-1)
      << "\n------\n";
}

int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    QStringList args = QCoreApplication::arguments();
    args.pop_front();
#ifdef QT_DIAG_LIB
    if (args.contains("-e")) {
        QtDiag::EventFilter::EventCategories cats = QtDiag::EventFilter::GeometryEvents
                | QtDiag::EventFilter::StateChangeEvents;
        a.installEventFilter(new QtDiag::EventFilter(cats, &a));
    }
#endif    

    qDebug() << QT_VERSION_STR << args
#if QT_VERSION >= 0x050000
          << QGuiApplication::platformName()
#endif
             ;

    MainWindow w;
    QDesktopWidget *dw = QApplication::desktop();
    const QRect ag = dw->availableGeometry();
    w.resize(ag.size() / 6);
    w.move(ag.center() - w.rect().bottomRight() / 2);
    w.show();
    w.dumpClicked();
#if QT_VERSION >= 0x050000
    QObject::connect(&a, &QGuiApplication::screenAdded,
                     &w, &MainWindow::screensAdded);
    QObject::connect(&a, &QGuiApplication::screenRemoved,
                     &w, &MainWindow::screensRemoved);
#endif
    QObject::connect(dw, SIGNAL(workAreaResized(int)),
                     &w, SLOT(workAreaResized(int)));
    QObject::connect(dw, SIGNAL(screenCountChanged(int)),
                     &w, SLOT(screenCountChanged(int)));
    return a.exec();
}
#include "main.moc"
